/*******************************************************************************



    CONTAGEM_REGRESSIVA

    Modo PWM - utiliza o PWM interno do pic a 19,53 khz.

    Autor: Claudio Lrios

    Data: 23/10/2017

    Microcontrolador: PIC 16f628a
    FLASH: 25P10  (usada como 'BIOS' de alguns notebook,pc,roteadores,etc)

    Objetivo: Ao apertar o boto 'Disp' comea a contagem regressiva
    sendo indicada no display e por voz no alto falante.
    Contagem regressiva de 10 segundos. 
    Sada de rele para disparo de foguetes experimentais.

    Parte integrante do blog PICSOURCE (LARIOS.TECNOLOGIA.WS)
    Proposta didtica apenas, sendo testada somente em placa de protoboard.

******************************************************************************/


#include <16F628A.h>
#use delay(clock=20000000)//USE XTAL DE 20MHZ
#fuses NOWDT,HS, NOPUT, NOPROTECT, BROWNOUT, NOMCLR, NOLVP, NOCPD
#use fast_io(a)
#use fast_io(b)
#ZERO_RAM
#rom 0x2100 = {0,0,0}

#byte trisa= 0x85
#byte trisb= 0x86
#byte porta= 0x05
#byte portb= 0x06
#byte pr2=   0x92
#byte ccp1l= 0x15

#bit    cs           = 0x05.1  //pino 18 ra1
#bit    miso         = 0x05.4  //pino 3 ra4
#bit    mosi         = 0x05.3  //pino 2 ra3
#bit    slk          = 0x05.2  //pino 1 ra2
#bit    flagt2       = 0x0C.1  //flag do timer 2
#bit    som          = 0x06.3  //pino 9 saida de som
#bit    tris_som     = 0x86.3  //sentido
#bit    tris_sda     = 0x86.0  //rb0 sentido
#bit    sw_disp      = 0x05.5  //ra5 pino4    disparo/abortar/reiniciar
#bit    rele         = 0x05.0  //pino 17 ra0
/******************************************************************************/
//Escolha aqui o tipo de display de 7 seguimentos: ctodo ou nodo comum:

    //     #define anodo  //comente para usar display 7 seg de catodo

/******************************************************************************/

#define VALOR_RESET 200  //valor que determina o tempo com sw disp apertado
                         //para resetar a contagem
                         
//variveis globais
unsigned int16 b;
unsigned int8 cnt,cnt1;//contadores auxiliares
unsigned int1 f_disp=0;//flag para indicar condio de contagem em andamento


//Conforme definido acima para display  nodo ou ctodo comum
#ifdef  anodo
//                    0     1   2    3    4     5    6   7    8    9    10
unsigned int8 num[]={0x80,0xf9,0x4c,0x68,0x39,0x2a,0x0a,0xf8,0x00,0x28,0x7f};//anodo
#else

//                      0     1   2    3    4     5    6   7    8    9    10
unsigned int8 num[]={0x7f,0x06,0xb3,0x97,0xc6,0xd5,0xf5,0x07,0xff,0xd7,0x80};//catodo
#endif
/*==============================================================================

                      ROTINAS DE COMUNICAO SPI

==============================================================================*/


/*==============================================================================
     GERA SINAL DE CLOCK PARA A FLASH
==============================================================================*/

void clock(void){ slk=1; delay_us(1);slk=0; delay_us(1);}

/*==============================================================================
    ENVIA 8 BITS PARA A FLASH
==============================================================================*/
void env8(int8 zbuf1){
  int8 a3;
  for(a3=0;a3<8;a3++){
  if(bit_test(zbuf1,7)) mosi=1; else mosi=0;
   clock();
   zbuf1<<=1;
  }
}
/*==============================================================================
     RECEBE 8 BITS DA FLASH
==============================================================================*/

int8 recebe_flash_8bits(void){
 int8 a5,b;
 b=0;
 for (a5=0;a5<8;a5++){
  shift_left(&b,1,miso); //original
 // shift_right(&b,1,miso);//teste
  clock();
 }
   return (b);
}


/*==============================================================================
     ROTINA DE REPRODUO DE SOM GRAVADO NA MEMRIA FLASH EXTERNA
==============================================================================*/

void reproduzir(int32 address){
 int8 buf;

  cs=0;
  env8(0x03);//comando de reproduzir em modo contnuo
  env8(make8(address,2));//pega endereo inicial
  env8(make8(address,1));
  env8(make8(address,0));
  tris_som=0;//port de som como sada
  for(;;){// reproduz at encontrar 0x22 no cdigo
      buf=recebe_flash_8bits(); //recebe byte
      if(buf==0x22) {/*tris_som=1; */cs=1;break;} //para no fim do arquivo
      if(!sw_disp)f_disp=0;//xxxxxxx
      while(!flagt2) ;//aguarda 'pwm' finalizar processo
      ccp1l=buf;//carga do valor do pwm
      flagt2=0;//zera flag tmr2
  }

  cs=1;
}

/*==============================================================================
         ROTINAS DE ENDEREAMENTO DA FLASH 25P10 PARA CADA SOM GRAVADO
==============================================================================*/          

void fala_zero(){
 reproduzir(0x1cb3a);//zero
}
void fala_um(){
 reproduzir(0x1b58e);//um
}

void fala_dois(){
reproduzir(0x183f9);//dois
}

void fala_tres(){
 reproduzir(0x158ca);//tres
}

void fala_quatro(){
reproduzir(0x12a9c);//quatro
}

void fala_cinco(){
reproduzir(0xf810);
}

void fala_seis(){
 reproduzir(0xc09c);
}

void fala_sete(){
 reproduzir(0x8e14);
}

void fala_oito(){
 reproduzir(0x6106);
}

void fala_nove(){
 reproduzir(0x30e4);
 }

void fala_dez(){
 reproduzir(0x00);
}


// pega um byte e o reproduz

void pega_num(int8 val){
switch (val){
    case 0: fala_zero();break;
    case 1: fala_um();break;
    case 2: fala_dois();break;
    case 3: fala_tres();break;
    case 4: fala_quatro();break;
    case 5: fala_cinco();break;//cinco
    case 6: fala_seis();break;//seis
    case 7: fala_sete();break;//sete
    case 8: fala_oito();break;//oito
    case 9: fala_nove();break;//nove
    case 10: fala_dez();break;//dez

    }
}

/*==============================================================================
                      ROTINA MAIN
==============================================================================*/

void main() {

   setup_counters(RTCC_INTERNAL,RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);
   setup_timer_2 ( T2_DIV_BY_1, 0, 1);
   setup_comparator(NC_NC_NC_NC);
   setup_ccp1(CCP_PWM);
   setup_vref(FALSE);
   trisa=0b11110000;
   trisb=0;
   porta=0; //desliga sadas
   portb=0;
   pr2=0xff;//periodo max
   cs=1;
   delay_ms(500);
   cnt=10;
/*==============================================================================

                      LOOP PRINCIPAL

==============================================================================*/
 for(;;){
   cnt1=0;//zera contagem
   if(!sw_disp){//testa 'sw_disp' quanto a disparo
     f_disp=!f_disp;//inverte entre disparado/abortado
     while(!sw_disp){//aguarda soltar interruptor
        delay_ms(10);
        if(++cnt1>VALOR_RESET) {//testa se deve resetar contagem 
          f_disp=0;//reseta flag de disparo
          cnt1=0;//zera contador auxiliar
          cnt=10;//volta a 'dez'
          portb=num[cnt];//mostra no display
        }
     }
   }
      
   while(f_disp){//se disparado, comea a contagem regressiva de 10 a 0
   
         portb=num[cnt];//converte hexa para 7 seguimentos
         if(!cnt){rele=1; f_disp=0;}//liga rele de disparo
         pega_num(cnt);// reproduz voz com o nmero apontado por cnt


         if(!cnt){//contagem chegou a zero?
          delay_ms(2000);//delay de 2 segundos com led aceso
           rele=0;//desliga rele
           f_disp=0;//sai da condio de disparado
           cnt=10;//mostra simbolo '-' para 'dez'

         }
         for(b=0;b<380;b++){//controla tempo de delay entre nmeros
            while(!sw_disp){
              f_disp=0;
              delay_ms(10);
              if(++cnt1>VALOR_RESET) {//testa se deve resetar contagem
                cnt1=0;
                cnt=10;
                portb=num[cnt];
                break;
              }
            }
         delay_ms(1);//delay
         }
         if(f_disp)cnt--;//se disparado decrementa cnt
    } //while(f_disp)
 
   
    

   }//for(;;)
}//main
